'=========================
'= Common Functions Code =
'=========================

'General functions that needed to go somewhere,
'which can probably be refactored!

'Written by Stuart Massey (VorTechS)
'3rd February 2010

'Some portions of this code have been taken from MSDN examples
'http://msdn.microsoft.com/en-us/library/bew39x2a(VS.71).aspx


'LICENSE INFORMATION:
'
'This file is part of the PhatHack LCD Driver software: you can redistribute it and/or modify
'it under the terms of the GNU General Public License as published by
'the Free Software Foundation, either version 3 of the License, or
'(at your option) any later version.

'This program is distributed in the hope that it will be useful,
'but WITHOUT ANY WARRANTY; without even the implied warranty of
'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
'GNU General Public License for more details.

'You should have received a copy of the GNU General Public License
'along with this program.  If not, see <http://www.gnu.org/licenses/>.

Imports System.Net.Sockets
Imports System.Net
Imports System.Text

Public Class Common

    Friend ConnectedClients As New List(Of DriverClient)
    Friend ConnectedDisplays As New List(Of CrystalFontz634)
    Friend _PortListener As Socket
    Friend Shared allDone As New Threading.ManualResetEvent(False)

    Public Sub SendClientText(ByVal Client As DriverClient, ByVal Text As String)

        If Not (Text.EndsWith(vbCrLf)) Then
            Text &= vbCrLf
        End If

        Call Send(Client.SourceSocket, Text)

    End Sub

    Public Sub Send(ByVal handler As Socket, ByVal data As String)

        If handler.Connected Then
            If Not (data.EndsWith(vbCrLf)) Then
                data &= vbCrLf
            End If

            ' Convert the string data to byte data using ASCII encoding.
            Dim byteData As Byte() = Encoding.ASCII.GetBytes(data)

            ' Begin sending the data to the remote device.
            handler.BeginSend(byteData, 0, byteData.Length, 0, New AsyncCallback(AddressOf SendCallback), handler)
        End If

    End Sub 'Send


    Private Sub SendCallback(ByVal ar As IAsyncResult)

        ' Retrieve the socket from the state object.
        Dim handler As Socket = CType(ar.AsyncState, Socket)

        If handler.Connected Then
            ' Complete sending the data to the remote device.
            Dim bytesSent As Integer = handler.EndSend(ar)
            Console.WriteLine("Sent {0} bytes to client.", bytesSent)
        End If

    End Sub 'SendCallback

    Public Function GetCrystalDisplayFromPort(ByVal Client As DriverClient) As CrystalFontz634

        Dim SourceDisplay As CrystalFontz634 = Nothing

        For Each availableDisplay As CrystalFontz634 In ConnectedDisplays
            If availableDisplay.Port = Client.DisplayPort Then
                SourceDisplay = availableDisplay
                Exit For
            End If
        Next availableDisplay

        Return SourceDisplay

    End Function

    Public Function GetKeyText(ByVal SourceText As String) As String

        Dim returnValue As String = SourceText

        returnValue = returnValue.Replace("{date}", Now.ToShortDateString)
        returnValue = returnValue.Replace("{time}", Now.ToLongTimeString)

        Return returnValue
    End Function

    Public Sub ShutDownClient(ByVal sourceClient As DriverClient)

        Call SendClientText(sourceClient, "Disconnection requested.  Shutting you down.")
        sourceClient.SourceSocket.Shutdown(SocketShutdown.Both)
        sourceClient.SourceSocket.Close()

        ConnectedClients.Remove(sourceClient)
        sourceClient = Nothing

    End Sub

    Public Sub StartListener()

        ' Establish the local endpoint for the socket.
        Dim ipHostInfo As Net.IPHostEntry = Dns.GetHostEntry("localhost")
        Dim ipAddress As Net.IPAddress = ipHostInfo.AddressList(0)
        Dim localEndPoint As New Net.IPEndPoint(ipAddress, 6456)

        ' Create a TCP/IP socket.
        _PortListener = New Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp)

        ' Bind the socket to the local endpoint and listen for incoming connections.
        _PortListener.Bind(localEndPoint)
        _PortListener.Listen(100)

        ' Start an asynchronous socket to listen for connections.
        Console.WriteLine("Waiting for a connection...")
        _PortListener.BeginAccept(New AsyncCallback(AddressOf AcceptCallback), _PortListener)

    End Sub

    Private Sub AcceptCallback(ByVal ar As IAsyncResult)

        ' Get the socket that handles the client request.
        Dim listener As Socket = CType(ar.AsyncState, Socket)

        ' End the operation.
        Dim handler As Socket = listener.EndAccept(ar)


        ' Create the state object for the async receive.
        Dim state As New AsynchronousSocketState

        state.workSocket = handler

        Dim newDriverClient = New DriverClient(state)
        Dim synchSocket As New AsynchronousSocketClient(newDriverClient)

        ConnectedClients.Add(newDriverClient)

        handler.BeginReceive(state.buffer, 0, AsynchronousSocketState.BufferSize, 0, New AsyncCallback(AddressOf synchSocket.ReadCallback), state)

        Call ClientConnected(newDriverClient)

        'Set the listener back to accepting a callback
        Console.WriteLine("Waiting for a connection...")
        _PortListener.BeginAccept(New AsyncCallback(AddressOf AcceptCallback), _PortListener)

    End Sub 'AcceptCallback

    Private Sub ClientConnected(ByVal sourceClient As DriverClient)

        Call SendClientText(sourceClient, "PhatHack CrystalFontz 634 LCD Driver")
        Call SendClientText(sourceClient, "Running version: " & My.Application.Info.Version.ToString())
        Call SendClientText(sourceClient, "")
        Call SendClientText(sourceClient, "Waiting for commands (type /HELP for help)")

    End Sub

    Public Function GetSpecialChars(ByVal SourceString As String, ByRef ReturnedString As String) As List(Of String)

        Dim SpecialCharsList = New List(Of String)
        Dim sourceParams() = Split(SourceString, "{")

        For Each sourceParam As String In sourceParams
            If sourceParam.Trim.EndsWith("}") Then
                If Not (sourceParam.ToLower.Contains("date")) And Not (sourceParam.ToLower.Contains("time")) Then
                    'Trim out the buffer, to remove the special characters which will be sent separately
                    ReturnedString = Replace(ReturnedString, "{" & sourceParam.Trim, "")
                    SpecialCharsList.Add(sourceParam.Trim.Replace("}", ""))
                End If
            End If
        Next sourceParam

        Return SpecialCharsList

    End Function
End Class
