'============================
'= CrystalFontz Packet Code =
'============================

'A wrapper class allowing commands
'to be structured separately from
'the parameters and data required for them.

'Contains some reference, although not implemented,
'to the required CRC for other display types.

'Written by Stuart Massey (VorTechS)
'3rd February 2010

'LICENSE INFORMATION:
'
'This file is part of the PhatHack LCD Driver software: you can redistribute it and/or modify
'it under the terms of the GNU General Public License as published by
'the Free Software Foundation, either version 3 of the License, or
'(at your option) any later version.

'This program is distributed in the hope that it will be useful,
'but WITHOUT ANY WARRANTY; without even the implied warranty of
'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
'GNU General Public License for more details.

'You should have received a copy of the GNU General Public License
'along with this program.  If not, see <http://www.gnu.org/licenses/>.

Public Class CrystalFontzPacket

    Public Structure WORD
        Dim Lo As Byte
        Dim Hi As Byte
    End Structure


    Private _dataCommand As Byte
    Private _dataCommandParams() As Byte
    Private _dataLength As Integer
    Private _data() As Byte
    Private _crc As WORD


    Public Sub New()

        'Standard constructor, allowing properties to be set individually

    End Sub

    Public Property DataCommand() As Byte
        Get
            Return _dataCommand
        End Get
        Set(ByVal value As Byte)
            _dataCommand = value
        End Set
    End Property

    Public Property DataCommandParams() As Byte()
        Get
            Return _dataCommandParams
        End Get
        Set(ByVal value As Byte())
            _dataCommandParams = value
        End Set
    End Property

    Public Property Data() As Byte()
        Get
            Return _data
        End Get
        Set(ByVal value As Byte())
            _data = value
        End Set
    End Property

    Public Function GetBytes() As Byte()

        'A message, at it's minimum can be 1 byte, excluding the command + CRC
        Dim MessageLen As Integer = 1

        'Now determine how long the message data really is - first any parameters
        'specified for the command?
        If Not (DataCommandParams Is Nothing) Then
            MessageLen += DataCommandParams.Length
        End If

        'What about data, any of that?
        If Not (Data Is Nothing) Then
            MessageLen += Data.Length
        End If

        'Now add two, for CRC based controlled LCDs
        'MessageLen += 1

        'Now build a single array to construct this packet message
        Dim PacketMessage(MessageLen - 1) As Byte
        Dim copyIndex As Integer = 1

        'Copy in the data command
        PacketMessage(0) = DataCommand

        'Now, the command parameters
        If Not (DataCommandParams Is Nothing) Then
            For nextIndex As Integer = 0 To DataCommandParams.Length - 1
                PacketMessage(copyIndex) = DataCommandParams(nextIndex)
                copyIndex += 1
            Next nextIndex
        End If

        'Now the data
        If Not (Data Is Nothing) Then
            For nextIndex As Integer = 0 To Data.Length - 1
                PacketMessage(copyIndex) = Data(nextIndex)
                copyIndex += 1
            Next nextIndex
        End If

        'Return this packet message
        Return PacketMessage

    End Function
End Class
