'===========================================
'= LCD Flashing Scrolling Line Source Code =
'===========================================

'A (Redundant) wrapper class for displaying a flashing and scrolling line on the LCD.

'Written by Stuart Massey (VorTechS)
'3rd February 2010

'LICENSE INFORMATION:
'
'This file is part of the PhatHack LCD Driver software: you can redistribute it and/or modify
'it under the terms of the GNU General Public License as published by
'the Free Software Foundation, either version 3 of the License, or
'(at your option) any later version.

'This program is distributed in the hope that it will be useful,
'but WITHOUT ANY WARRANTY; without even the implied warranty of
'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
'GNU General Public License for more details.

'You should have received a copy of the GNU General Public License
'along with this program.  If not, see <http://www.gnu.org/licenses/>.

Public Class FlashScroller

    Private _flashingString As String = String.Empty
    Private _bufferedString As String = String.Empty
    Private _originalString As String = String.Empty
    Private _destinationLine As Integer = -1
    Private _destinationColumn As Integer = -1
    Private _FlashOn As Boolean
    Private _sourceDisplay As CrystalFontz634 = Nothing
    Private _scrollPosition As Integer = 0

    Public FlashDelay As Integer

    Private WithEvents _scrollTimer As System.Timers.Timer

    Public Sub New()

    End Sub

    Public Sub New(ByVal FlashingText As String, ByVal Line As Integer, ByVal Column As Integer, ByVal FlashDelay As Integer, ByVal SourceDisplay As CrystalFontz634)

        'The flashing text is the static part of the final string, that flashes on and off
        Me.FlashingText = FlashingText

        Me.FlashDelay = FlashDelay
        Me.DestinationLine = Line

    End Sub

    Public Property ScrollText() As String
        Get
            Return _originalString
        End Get
        Set(ByVal value As String)
            _originalString = value
            _bufferedString = Space(20) + value + Space(1)
        End Set
    End Property

    Public Property FlashingText() As String
        Get
            Return _flashingString
        End Get
        Set(ByVal value As String)
            _flashingString = value
        End Set
    End Property

    Public Property DestinationLine() As Integer
        Get
            Return _destinationLine
        End Get
        Set(ByVal value As Integer)
            _destinationLine = value
        End Set
    End Property

    Public Property DestinationColumn() As Integer
        Get
            Return _destinationColumn
        End Get
        Set(ByVal value As Integer)
            _destinationColumn = value
        End Set
    End Property

    Public Property SourceDisplay() As CrystalFontz634
        Get
            Return _sourceDisplay
        End Get
        Set(ByVal value As CrystalFontz634)
            _sourceDisplay = value
        End Set
    End Property

    Public Sub Start()

        If Not (_scrollTimer Is Nothing) Then
            _scrollTimer.Stop()
            _scrollTimer = Nothing
        End If

        If FlashDelay > 0 And Not (_sourceDisplay Is Nothing) Then
            _scrollTimer = New System.Timers.Timer(FlashDelay)
            _scrollTimer.Start()
        End If

    End Sub

    Private Sub _scrollTimer_Elapsed(ByVal sender As Object, ByVal e As System.Timers.ElapsedEventArgs) Handles _scrollTimer.Elapsed

        _scrollTimer.Stop()

        _FlashOn = Not (_FlashOn)

        'Use a copy of the flashing string (so it's not lost)
        Dim sourceSplashingString = _flashingString

        'And determine the special characters to be displayed
        Dim SpecialChars As List(Of String) = CommonFunctions.GetSpecialChars(_flashingString, sourceSplashingString)

        Dim destColOffset As Integer = 0
        Dim StringAdjustmentCount As Integer = 0

        Dim DisplayString As String = ""
        Dim CharactersSent As Integer = 0

        'Firstly, set the cursor position
        Call _sourceDisplay.SetCursorPosition(_destinationLine, _destinationColumn)


        'We need to send the display string in a controlled manner, character by character
        Dim IsSpecialCharOpen As Boolean = False
        Dim loopSourceString As String = ""

        For StringLoop As Integer = 0 To 1


            If StringLoop = 0 Then
                loopSourceString = _flashingString
            Else
                If (_scrollPosition + (20 - CharactersSent)) < _bufferedString.Length Then
                    loopSourceString = _bufferedString.Substring(_scrollPosition, 20 - CharactersSent)
                Else
                    loopSourceString = _bufferedString.Substring(_scrollPosition)
                End If
            End If

            For Each textChar As String In loopSourceString
                Dim SendIt As Boolean = True

                If (textChar = "{") And Not (IsSpecialCharOpen) Then
                    SendIt = False
                    IsSpecialCharOpen = True
                End If

                If (textChar = "}") And (IsSpecialCharOpen) Then
                    SendIt = False
                    IsSpecialCharOpen = False
                End If

                If (SendIt) Then
                    If (IsSpecialCharOpen) And _FlashOn Then
                        'Send the special character to the display
                        _sourceDisplay.DirectWriteSpecialChar(textChar)
                    ElseIf _FlashOn Or StringLoop = 1 Then
                        'Write this character directly to the display
                        _sourceDisplay.DirectWrite(System.Text.ASCIIEncoding.ASCII.GetBytes(textChar))
                    Else
                        'Write this character directly to the display
                        _sourceDisplay.DirectWrite(System.Text.ASCIIEncoding.ASCII.GetBytes(" "))
                    End If

                    'Keep a count of the characters sent....
                    CharactersSent += 1
                End If

            Next textChar
        Next StringLoop

        _scrollPosition += 1

        If _scrollPosition = _bufferedString.Length + 1 Then
            _scrollPosition = 0
        End If

        _scrollTimer.Start()

    End Sub

    Public Sub StopFlashing()

        _scrollTimer.Stop()
        _scrollTimer = Nothing

    End Sub


End Class

