'=================================
'= LCD Flashing Line Source Code =
'=================================

'A (Redundant) wrapper class for displaying a flashing line on the LCD.

'Written by Stuart Massey (VorTechS)
'3rd February 2010

'LICENSE INFORMATION:
'
'This file is part of the PhatHack LCD Driver software: you can redistribute it and/or modify
'it under the terms of the GNU General Public License as published by
'the Free Software Foundation, either version 3 of the License, or
'(at your option) any later version.

'This program is distributed in the hope that it will be useful,
'but WITHOUT ANY WARRANTY; without even the implied warranty of
'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
'GNU General Public License for more details.

'You should have received a copy of the GNU General Public License
'along with this program.  If not, see <http://www.gnu.org/licenses/>.

Public Class Flasher

    Private _bufferedString As String = String.Empty
    Private _destinationLine As Integer = -1
    Private _destinationColumn As Integer = -1
    Private _FlashOn As Boolean
    Private _sourceDisplay As CrystalFontz634 = Nothing

    Public FlashDelay As Integer

    Private WithEvents _scrollTimer As System.Timers.Timer

    Public Sub New()

    End Sub

    Public Sub New(ByVal FlashingText As String, ByVal Line As Integer, ByVal Column As Integer, ByVal FlashDelay As Integer, ByVal SourceDisplay As CrystalFontz634)

        Me.FlashingText = FlashingText
        Me.FlashDelay = FlashDelay
        Me.DestinationLine = Line

    End Sub

    Public Property FlashingText() As String
        Get
            Return _bufferedString
        End Get
        Set(ByVal value As String)
            _bufferedString = value
        End Set
    End Property

    Public Property DestinationLine() As Integer
        Get
            Return _destinationLine
        End Get
        Set(ByVal value As Integer)
            _destinationLine = value
        End Set
    End Property

    Public Property DestinationColumn() As Integer
        Get
            Return _destinationColumn
        End Get
        Set(ByVal value As Integer)
            _destinationColumn = value
        End Set
    End Property

    Public Property SourceDisplay() As CrystalFontz634
        Get
            Return _sourceDisplay
        End Get
        Set(ByVal value As CrystalFontz634)
            _sourceDisplay = value
        End Set
    End Property

    Public Sub Start()

        If Not (_scrollTimer Is Nothing) Then
            _scrollTimer.Stop()
            _scrollTimer = Nothing
        End If

        If FlashDelay > 0 And Not (_sourceDisplay Is Nothing) Then
            _scrollTimer = New System.Timers.Timer(FlashDelay)
            _scrollTimer.Start()
        End If

    End Sub

    Private Sub _scrollTimer_Elapsed(ByVal sender As Object, ByVal e As System.Timers.ElapsedEventArgs) Handles _scrollTimer.Elapsed

        _scrollTimer.Stop()

        _FlashOn = Not (_FlashOn)

        If (_FlashOn) Then
            'Send the string to the display
            _sourceDisplay.SendText(_destinationLine, _destinationColumn, _bufferedString)
        Else
            _sourceDisplay.SendText(_destinationLine, _destinationColumn, Space(20))
        End If

        _scrollTimer.Start()

    End Sub

    Public Sub StopFlashing()

        _scrollTimer.Stop()
        _scrollTimer = Nothing

    End Sub


End Class

