'=================================
'= LCD Text Scroller Source Code =
'=================================

'A (Redundant) wrapper class for displaying a scrolling line on the LCD.

'Written by Stuart Massey (VorTechS)
'3rd February 2010

'LICENSE INFORMATION:
'
'This file is part of the PhatHack LCD Driver software: you can redistribute it and/or modify
'it under the terms of the GNU General Public License as published by
'the Free Software Foundation, either version 3 of the License, or
'(at your option) any later version.

'This program is distributed in the hope that it will be useful,
'but WITHOUT ANY WARRANTY; without even the implied warranty of
'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
'GNU General Public License for more details.

'You should have received a copy of the GNU General Public License
'along with this program.  If not, see <http://www.gnu.org/licenses/>.

Public Class TextScroller

    Private _bufferedString As String = String.Empty
    Private _originalString As String = String.Empty
    Private _destinationLine As Integer = -1
    Private _scrollPosition As Integer
    Private _sourceDisplay As CrystalFontz634 = Nothing

    Public ScrollDelay As Integer

    Private WithEvents _scrollTimer As System.Timers.Timer

    Public Sub New()

    End Sub

    Public Sub New(ByVal ScrollText As String, ByVal Line As Integer, ByVal ScrollDelay As Integer, ByVal SourceDisplay As CrystalFontz634)

        Me.ScrollText = ScrollText
        Me.ScrollDelay = ScrollDelay
        Me.DestinationLine = Line

    End Sub

    Public Property ScrollText() As String
        Get
            Return _originalString
        End Get
        Set(ByVal value As String)
            _originalString = value
            _bufferedString = Space(20) + value + Space(1)
        End Set
    End Property

    Public Property DestinationLine() As Integer
        Get
            Return _destinationLine
        End Get
        Set(ByVal value As Integer)
            _destinationLine = value
        End Set
    End Property

    Public Property SourceDisplay() As CrystalFontz634
        Get
            Return _sourceDisplay
        End Get
        Set(ByVal value As CrystalFontz634)
            _sourceDisplay = value
        End Set
    End Property

    Public Sub Start()

        If Not (_scrollTimer Is Nothing) Then
            _scrollTimer.Stop()
            _scrollTimer = Nothing
        End If

        If ScrollDelay > 0 And Not (_sourceDisplay Is Nothing) Then
            _scrollTimer = New System.Timers.Timer(ScrollDelay)
            _scrollTimer.Start()
        End If

    End Sub

    Private Sub _scrollTimer_Elapsed(ByVal sender As Object, ByVal e As System.Timers.ElapsedEventArgs) Handles _scrollTimer.Elapsed

        _scrollTimer.Stop()

        'Get the 20 characters for current display
        Dim DisplayString As String = ""

        If (_scrollPosition + 20) < _bufferedString.Length Then
            DisplayString = _bufferedString.Substring(_scrollPosition, 20)
        Else
            DisplayString = _bufferedString.Substring(_scrollPosition)
        End If

        'Send the string to the display
        _sourceDisplay.SendText(_destinationLine, 0, DisplayString)

        _scrollPosition += 1

        If _scrollPosition = _bufferedString.Length + 1 Then
            _scrollPosition = 0
        End If

        _scrollTimer.Start()

    End Sub

    Public Sub StopScroll()

        If Not (_scrollTimer Is Nothing) Then
            _scrollTimer.Stop()
            _scrollTimer = Nothing
        End If

    End Sub


End Class
