'================================
'= Sample application Form Code =
'================================

'Makes use of the CrystalFontz service
'to demonstrate TCP -> RS232 connectivity
'and a simplified instruction set on getting
'things displayed on the LCD.

'Written by Stuart Massey (VorTechS)
'(C) 3rd February 2010

'NOTE: The LCD Driver project is designed as a Windows Service, 
'with all of the associated features ready for deployment.

'It is not necessary to install the service in order to start using
'this sample application - as the sample application has a reference
'embedded to the project, and deliberately starts the 'service listener'
'as if it had been running as a service. (Call commonstuff.StartListener)


'LICENSE INFORMATION:
'
'This file is part of the PhatHack LCD Driver software: you can redistribute it and/or modify
'it under the terms of the GNU General Public License as published by
'the Free Software Foundation, either version 3 of the License, or
'(at your option) any later version.

'This program is distributed in the hope that it will be useful,
'but WITHOUT ANY WARRANTY; without even the implied warranty of
'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
'GNU General Public License for more details.

'You should have received a copy of the GNU General Public License
'along with this program.  If not, see <http://www.gnu.org/licenses/>.

Public Class Form1

    Dim WithEvents DateWatcher As System.Timers.Timer
    Dim lcdClient As Net.Sockets.TcpClient
    Dim commonstuff As New PhatHack_LCD_Driver.Common

    Dim StartedDemo As Boolean = False
    Dim StartTime As DateTime = Nothing
    Dim timerTicks As Integer = 0

    Private Delegate Sub SubInvoker()

    Private Sub DateWatcher_Elapsed(ByVal sender As Object, ByVal e As System.Timers.ElapsedEventArgs) Handles DateWatcher.Elapsed

        'Stop the timer, looks fairly pointless, but as timers are threaded stopping it now, stops any further
        'updates so that the display is managed in a more controlled manner
        DateWatcher.Stop()

        If (StartedDemo) Then
            'Increment the timer ticks... we will use this to determine what updates to perform, and when
            timerTicks += 1

            Select Case timerTicks
                'Comment this case line out, and the one below in to show a variation on updating
                'the display scrolling using a slower speed
                'Case 1, 2, 3, 4
                Case 1, 3
                    'Update the scrolling positions
                    Call SendClientText(lcdClient, "scroll /line 0")
                    Call SendClientText(lcdClient, "scroll /line 1")

                Case 5
                    'Half a second.... let's update the flashing and times on line 3
                    Call SendClientText(lcdClient, "flash /line 3")

                    'Work out how long has passed since starting the demo using the 'hh:mm:ss' format
                    Dim ElapsedString As String = Format(DateDiff(DateInterval.Hour, StartTime, Now) Mod 60, "00") & ":" & Format(DateDiff(DateInterval.Minute, StartTime, Now) Mod 60, "00") & ":" & Format(DateDiff(DateInterval.Second, StartTime, Now) Mod 60, "00")

                    'Send a command to update the content of our display 'style'.  /Static <x> = update static element <x>, 
                    '/Scroll <x> = Update scrolling element <x> and /Flash <x> = update flashing element <x>
                    Call SendClientText(lcdClient, "content /line 3 /static 0 /value {time} ")
                    Call SendClientText(lcdClient, "content /line 3 /static 1 /value " & ElapsedString)

            End Select

            'Without this, you won't get much on the display!  This forces the service to go ahead and update
            'the display with the buffered content
            Call SendClientText(lcdClient, "update")

            'Reset the timer ticks once we've got to 500 milliseconds
            If timerTicks = 5 Then
                timerTicks = 0
            End If
        End If

        'Because there is threading involved - it will be necessary to check who currently owns
        'this thread, and make sure we modify the control on the correct thread
        Call CheckClientData()

        'Start the timer again
        DateWatcher.Start()

    End Sub

    Private Sub CheckClientData()

        If Not (lcdClient Is Nothing) Then
            If Me.InvokeRequired Then
                Me.Invoke(New SubInvoker(AddressOf CheckClientData))
            Else
                If (lcdClient.Available) Then
                    lblStatus.Text = GetClientText(lcdClient)
                End If
            End If
        End If

    End Sub

    Public Sub SendClientText(ByVal Client As Net.Sockets.TcpClient, ByVal Text As String)

        If Not (Text.EndsWith(vbCrLf)) Then
            Text &= vbCrLf
        End If

        Dim bytes() As Byte = System.Text.Encoding.ASCII.GetBytes(Text)

        Call Client.GetStream.Write(bytes, 0, bytes.Length)

    End Sub

    Public Function GetClientText(ByVal Client As Net.Sockets.TcpClient)

        Dim buffer(Client.Available) As Byte
        Call Client.GetStream.Read(buffer, 0, buffer.Length)

        Return System.Text.ASCIIEncoding.ASCII.GetString(buffer)

    End Function

    Private Sub Form1_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load

        'Evaluate the available COM ports
        cmbPorts.Items.Clear()

        Dim ports() As String = IO.Ports.SerialPort.GetPortNames()

        For Each Port As String In ports
            cmbPorts.Items.Add(Port)
        Next Port

        If cmbPorts.Items.Count > 0 Then
            cmbPorts.SelectedIndex = 0
        End If

        cmbBaudRate.SelectedIndex = 0
        cmbParity.SelectedIndex = 0
        cmbStopBits.SelectedIndex = 1
        cmbDataBits.SelectedIndex = cmbDataBits.Items.Count - 1

        'Fire up the listener, as we need to debug in non-service mode
        Try
            'If this fails, stop the PhatHack service list under the services....
            Call commonstuff.StartListener()
        Catch ex As Exception

        End Try
    End Sub

    Private Sub btnConnect_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnConnect.Click

        If btnConnect.Tag = 0 Then
            Cursor = Cursors.WaitCursor

            btnConnect.Tag = 1
            btnDisplay.Enabled = True
            btnConnect.Text = btnConnect.Text.Replace("CONNECT", "DISCONNECT")

            System.Threading.Thread.Sleep(1000)

            lcdClient = New Net.Sockets.TcpClient

            If (DateWatcher Is Nothing) Then

                'Get the relevant parameters
                Dim Port As String = cmbPorts.SelectedItem.ToString.ToUpper.Replace("COM", "")
                Dim Baud As String = cmbBaudRate.SelectedItem.ToString
                Dim Parity As String = cmbParity.SelectedIndex.ToString

                Dim StopBits As String = ""
                Dim DataBits As String = cmbDataBits.SelectedItem.ToString

                If cmbStopBits.SelectedIndex = 2 Then
                    StopBits = "3"
                ElseIf cmbStopBits.SelectedIndex = 3 Then
                    StopBits = "2"
                Else
                    StopBits = cmbStopBits.SelectedItem.ToString()
                End If

                Dim dtrValue As Integer = IIf(chkDTREnabled.Checked, 1, 0)
                Dim rtsValue As Integer = IIf(chkRTSEnabled.Checked, 1, 0)

                Call lcdClient.Connect("localhost", 6456)
                If lcdClient.Connected Then

                    'Set up the LCD communication - using COM port 1
                    Call SendClientText(lcdClient, "start /port " & Port & " /baud " & Baud & " /parity " & Parity & " /stopbits " & StopBits & " /databits " & DataBits & " /dtrOn " & dtrValue & " /rtsOn " & rtsValue)

                    'Wait for a second to allow the display to be initialised
                    System.Threading.Thread.Sleep(1000)

                    'Now clear the display
                    Call SendClientText(lcdClient, "clear")
                End If
            End If
            Cursor = Cursors.Default

        Else
            Cursor = Cursors.WaitCursor

            btnConnect.Tag = 0
            btnDisplay.Enabled = False

            If StartedDemo Then
                Call btnDisplay_Click(Nothing, Nothing)
            End If
            btnConnect.Text = btnConnect.Text.Replace("DISCONNECT", "CONNECT")

            Call SendClientText(lcdClient, "disconnect")
            lcdClient = Nothing

            Cursor = Cursors.Default
        End If

        Call CheckClientData()

    End Sub

    Private Sub btnDisplay_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnDisplay.Click

        If btnDisplay.Tag = 0 Then
            btnDisplay.Tag = 1

            'Set the start point to now for the timer to evaluate
            'into an elapsed time
            StartTime = Now

            'Send commands to initialise a multi-style display on lines 1 & 2 of the display
            '[0]=Static, [1]=Flash, [2*xx]=Scroll, displaying <xx> characters
            Call SendClientText(lcdClient, "content /line 0 /style [0]Album: [2*13]")
            Call SendClientText(lcdClient, "content /line 1 /style [0]Track: [2*13]")

            'Call SendClientText(lcdClient, "content /line 0 /style [0]Album: [2*13]The Prodigy: Experience....")
            'Call SendClientText(lcdClient, "content /line 1 /style [0]Track: [2*13]Everybody in the Place (Let's Go!)")

            'Updates the specified multi-style line display with a new value (NOTE: these two commands were NOT
            'actually necesssary for the purposes of the demo.  You could have specified the values in-place
            'as the two lines above demonstrate
            Call SendClientText(lcdClient, "content /line 0 /scroll 0 /value The Prodigy: Experience....")
            Call SendClientText(lcdClient, "content /line 1 /scroll 0 /value Everybody in the Place (Let's Go!)")

            'Configure line 4 to display a style with a (static) 'current time' display and a flashing 'play' symbol
            Call SendClientText(lcdClient, "content /line 3 /style [0] {time}  [1] {16} [0]00:00 ")
            StartedDemo = True
            btnDisplay.Text = btnDisplay.Text.Replace("START", "STOP")

            'Create a new timer firing it every <xx> milliseconds
            'and don't forget to start it - or nothing exciting will happen!
            DateWatcher = New System.Timers.Timer(100)
            DateWatcher.Start()

        Else
            Call SendClientText(lcdClient, "reset")

            DateWatcher.Stop()
            DateWatcher = Nothing

            StartedDemo = False
            btnDisplay.Text = btnDisplay.Text.Replace("STOP", "START")
            btnDisplay.Tag = 0
        End If

    End Sub
End Class
